package com.tofly.watermeter.custom.mapper.meterBusiness;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.InstallConditionDto;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.InstallTableDto;
import com.tofly.watermeter.entity.Qfkhjbxx;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

public interface TableInstallMapper {

    Page<Map> getCustomerInfo(Page page,@Param("dto") String dto);

    Page<Map> getTableInfo(Page page, @Param("dto") String dto, @Param("sbcj") Long sbcj, @Param("sblx") Long sblx, @Param("sbkj") Long sbkj);

    List<Map> installCount(@Param("dto") InstallConditionDto dto);

    List<Map> getGdInfo(@Param("workOrderCode") List<String> workOrderCode);

    Page<Map> countByNewTable(Page page, @Param("dto") InstallConditionDto dto);

    List<Map> countByNewTable(@Param("dto") InstallConditionDto dto);

    List<Map> theStagingQuery(@Param("dto") InstallTableDto dto);

    Page<Map> theStagingQuery(Page page,@Param("dto") InstallTableDto dto);

    List<Qfkhjbxx> queryClientInfo(@Param("code") List<String> code);

    List<String> queryInstallTable(@Param("code") List<String> code);
}
